using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Net;
using System.Text;
using System.Windows.Forms;
using System.Xml;

using Fusion8.Cropper.Extensibility;

namespace Cropper.SendToTwitPic
{
    public class TwitPic : IPersistableImageFormat, IConfigurablePlugin
    {
        const string CROPPER_TWITPIC_EVENT_SOURCE_NAME = "Cropper-TwitPic";
        const string TWITPIC_UPLOAD_URL = "http://twitpic.com/api/upload";

        private const bool hostInOptions = true;
        private TwitPicSettings _settings;
        Options _configurationForm;
        private IPersistableOutput _output;

        //private string 

        public TwitPic()
        {
            if (!EventLog.SourceExists(CROPPER_TWITPIC_EVENT_SOURCE_NAME))
                EventLog.CreateEventSource(CROPPER_TWITPIC_EVENT_SOURCE_NAME, "Application");
        }

        #region Properties

        private string _fileName = string.Empty;
        public string FileName
        {
            get { return _fileName; }
            set { _fileName = value; }
        }

        private string _imageFileContents = string.Empty;
        public string ImageFileContents
        {
            get { return _imageFileContents; }
            set { _imageFileContents = value; }
        }

        private string _postedMediaUrl = string.Empty;
        public string PostedMediaUrl
        {
            get { return _postedMediaUrl; }
            set { _postedMediaUrl = value; }
        }

        private string _twitPicErrorMsg = string.Empty;
        public string TwitPicErrorMsg
        {
            get { return _twitPicErrorMsg;  }
            set { _twitPicErrorMsg = value; }
        }

        private string _twitPicPostingUrl = TWITPIC_UPLOAD_URL;
        public string TwitPicPostingUrl
        {
            get { return _twitPicPostingUrl; }
            set { _twitPicPostingUrl = value; }
        }

        private string _twitPicRequest = string.Empty;
        public string TwitPicRequest
        {
            get { return _twitPicRequest; }
            set { _twitPicRequest = value; }
        }

        #endregion

        #region IConfigurablePlugin Implementation

        /// <summary>
        /// Gets the plug-ins impementation of the <see cref="BaseConfigurationForm"/> used 
        /// for setting plug-in specific options.
        /// </summary>
        public BaseConfigurationForm ConfigurationForm
        {
            get
            {
                if (_configurationForm == null)
                {
                    _configurationForm = new Options();
                    _configurationForm.OptionsSaved += Handle_configurationFormOptionsSaved;
                    _configurationForm.TwitPicID       = PluginSettings.TwitPicID;
                    _configurationForm.TwitPicPassword = PluginSettings.TwitPicPassword;
                    _configurationForm.ShowDebugInfo   = PluginSettings.ShowDebugInfo;
                }
                return _configurationForm;
            }
        }

        /// <summary>
        /// Gets a value indicating if the <see cref="_configurationForm"/> is to be hosted 
        /// in the options dialog or shown in its own dialog window.
        /// </summary>
        public bool HostInOptions
        {
            get { return hostInOptions; }
        }

        /// <summary>
        /// Gets or sets an object containing the plug-in's _settings.
        /// </summary>
        /// <remarks>
        /// <para>
        /// This property is set during startup with the _settings contained in the applications
        /// configuration file.</para>
        /// <para>
        /// The object returned by this property is serialized into the applications configuration
        /// file on shutdown.</para></remarks>
        public object Settings
        {
            get { return PluginSettings; }
            set { PluginSettings = value as TwitPicSettings; }
        }

        // Helper property for IConfigurablePlugin Implementation
        public TwitPicSettings PluginSettings
        {
            get
            {
                if (_settings == null)
                    _settings = new TwitPicSettings();
                return _settings;
            }
            set { _settings = value; }
        }

        private void Handle_configurationFormOptionsSaved(object sender, EventArgs e)
        {
            PluginSettings.TwitPicID       = _configurationForm.TwitPicID;
            PluginSettings.TwitPicPassword = _configurationForm.TwitPicPassword;
            PluginSettings.ShowDebugInfo   = _configurationForm.ShowDebugInfo;
        }

        #endregion

        #region IPersistableImageFormat Members

        public string Description
        {
            get { return "Send screenshot to your TwitPic account."; }
        }

        public string Extension
        {
            get { return "Png"; }
        }

        public IPersistableImageFormat Format
        {
            get { return this; }
        }

        public override string ToString()
        {
            return "Send to your TwitPic account";
        }

        public void Connect(IPersistableOutput persistableOutput)
        {
            if (persistableOutput == null)
            {
                throw new ArgumentNullException("persistableOutput");
            }
            _output = persistableOutput;
            _output.ImageCaptured  += new ImageCapturedEventHandler(this.persistableOutput_ImageCaptured);
            _output.ImageCapturing += new ImageCapturingEventHandler(this.persistableOutput_ImageCapturing);
        }

        public void Disconnect()
        {
            _output.ImageCaptured -= new ImageCapturedEventHandler(this.persistableOutput_ImageCaptured);
            _output.ImageCapturing -= new ImageCapturingEventHandler(this.persistableOutput_ImageCapturing);
        }

        public event ImageFormatClickEventHandler ImageFormatClick;

        private void OnImageFormatClick(object sender, ImageFormatEventArgs e)
        {
            if (ImageFormatClick != null)
            {
                ImageFormatClick(sender, e);
            }
        }

        void persistableOutput_ImageCaptured(object sender, ImageCapturedEventArgs e)
        {
            if (string.IsNullOrEmpty(this.PluginSettings.TwitPicID) ||
                string.IsNullOrEmpty(this.PluginSettings.TwitPicPassword))
            {
                string setupInformation = "Please configure your TwitPic _settings on the Cropper Options / Plug-ins tab.\n\n" +
                    "For information on these _settings, see http://twitpic.com\n";
                MessageBox.Show(setupInformation, "Missing TwitPic Settings");
                return;
            }

            try
            {
                ImagePairNames imgNames = e.ImageNames;
                this.FileName = e.ImageNames.FullSize;
                this.LogDebugInfo(string.Format("Converting the captured image to .png - file name: {0}", this.FileName));

                // Convert the captured image as .png and string encode it so we can upload it
                MemoryStream imageStream = new MemoryStream();
                e.FullSizeImage.Save(imageStream, System.Drawing.Imaging.ImageFormat.Png);
                imageStream.Position = 0;
                byte[] data = imageStream.ToArray();
                this.ImageFileContents = Encoding.GetEncoding("iso-8859-1").GetString(data);

                this.PostToTwitPic();
                MessageBox.Show(string.Format("Your image is available on TwitPic at\n{0}",this.PostedMediaUrl),
                                "Cropper.SendToTwitPic");
            }
            catch (Exception ex)
            {
                EventLog.WriteEntry(CROPPER_TWITPIC_EVENT_SOURCE_NAME, 
                                    "An exception occurred", 
                                    EventLogEntryType.Error, 
                                    1,
                                    1,
                                    Encoding.GetEncoding("iso-8859-1").GetBytes(ex.ToString() + ex.StackTrace));
                MessageBox.Show(ex.Message, "An exception occurred - see the Windows Application Event Log for details");
            }
        }


        void persistableOutput_ImageCapturing(object sender, ImageCapturingEventArgs e)
        {
            this.LogDebugInfo("Capturing an image via Cropper-TwitPic");
        }

        public MenuItem Menu
        {
            get
            {
                MenuItem item = new MenuItem();
                item.RadioCheck = true;
                item.Text = "Send to TwitPic";
                item.Click += new EventHandler(this.MenuItemClick);
                return item;
            }
        }

        private void MenuItemClick(object sender, EventArgs e)
        {
            ImageFormatEventArgs args = new ImageFormatEventArgs();
            args.ClickedMenuItem = (MenuItem)sender;
            args.ImageOutputFormat = this;
            this.OnImageFormatClick(sender, args);
        }


        #endregion

        #region TwitPic Interface

        public void PostToTwitPic()
        {
            // Reset TwitPic output
            this.PostedMediaUrl = string.Empty;
            this.TwitPicErrorMsg = string.Empty;

            // Create the TwitPic request
            string formDataBoundary = DateTime.Now.Ticks.ToString("x");
            string formDataEncoding = "iso-8859-1";

            this.TwitPicRequest = this.CreateRequestContents(formDataBoundary);
            byte[] bytes = Encoding.GetEncoding(formDataEncoding).GetBytes(this.TwitPicRequest);
            
            // Initialize the request for TwitPic
            HttpWebRequest request = (HttpWebRequest)WebRequest.Create(this.TwitPicPostingUrl);
            request.Method = "POST";
            request.CookieContainer = null;
            request.Proxy.Credentials = CredentialCache.DefaultCredentials;
            request.PreAuthenticate = true;
            request.AllowWriteStreamBuffering = true;
            request.ContentType = string.Format("multipart/form-data; boundary={0}", formDataBoundary);
            request.ContentLength = bytes.Length;

            // Send to TwitPic
            using (Stream requestStream = request.GetRequestStream())
            {
                // Post the data
                this.LogDebugInfo(string.Format("Uploading the captured image to: {0}", this.TwitPicPostingUrl));
                requestStream.Write(bytes, 0, bytes.Length);

                // Get the response
                using (HttpWebResponse response = (HttpWebResponse)request.GetResponse())
                {
                    this.LogDebugInfo(string.Format("Getting the upload status from TwitPic"));

                    using (StreamReader reader = new StreamReader(response.GetResponseStream()))
                    {
                        string result = reader.ReadToEnd();
                        XmlDocument docResult = new XmlDocument();
                        docResult.LoadXml(result);
                        XmlNode nodeResult = docResult.SelectSingleNode("/rsp");
                        if (nodeResult == null)
                            EventLog.WriteEntry(CROPPER_TWITPIC_EVENT_SOURCE_NAME, 
                                                "Did not receive a valid status response from TwitPic", 
                                                EventLogEntryType.Error, 
                                                2);
                        else
                        {
                            string resultCode = nodeResult.Attributes["stat"].Value;
                            if ("ok" == resultCode)
                            {
                                XmlNode nodeMediaUrl = docResult.SelectSingleNode("//mediaurl");
                                this.PostedMediaUrl = (nodeMediaUrl != null) ? nodeMediaUrl.InnerText : "unknown media url";
                                this.LogDebugInfo(string.Format("TwitPic media url: {0}", this.PostedMediaUrl));
                            }
                            else
                            {
                                XmlNode nodeError = docResult.SelectSingleNode("//err");
                                this.TwitPicErrorMsg = (nodeError != null) 
                                                        ? string.Format("Code:{0} Msg: {1}",
                                                                        nodeError.Attributes["code"].Value,
                                                                        nodeError.Attributes["msg"].Value)
                                                        : "unknown error message";
                                EventLog.WriteEntry(CROPPER_TWITPIC_EVENT_SOURCE_NAME,
                                                    string.Format("TwitPic error: {0}", this.TwitPicErrorMsg), 
                                                    EventLogEntryType.Error, 
                                                    3);
                            }
                        }
                    }
                }
            }
        }

        public string CreateRequestContents(string formDataBoundary)
        {
            string formDataHeader = string.Format("--{0}", formDataBoundary);
            string formDataFooter = string.Format("--{0}--", formDataBoundary);

            StringBuilder contents = new StringBuilder();
            contents.AppendLine(formDataHeader);

            // Add "media" field
            contents.AppendLine(String.Format("Content-Disposition: file; name=\"media\"; filename=\"{0}\"", this.FileName));
            contents.AppendLine("Content-Type: image/png");
            contents.AppendLine();
            contents.AppendLine(this.ImageFileContents);

            // Add username field
            contents.AppendLine(formDataHeader);
            contents.AppendLine("Content-Disposition: form-data; name=\"username\"");
            contents.AppendLine();
            contents.AppendLine(this.PluginSettings.TwitPicID);

            // Add password field
            contents.AppendLine(formDataHeader);
            contents.AppendLine("Content-Disposition: form-data; name=\"password\"");
            contents.AppendLine();
            contents.AppendLine(this.PluginSettings.TwitPicPassword);

            contents.AppendLine(formDataFooter);

            string stringContents = contents.ToString();

            this.LogDebugInfo("Formatted the TwitPic upload request",
                                Encoding.GetEncoding("iso-8859-1").GetBytes(stringContents));
            return stringContents;
        }

        #endregion

        #region Debugging

        private void LogDebugInfo(string msg)
        {
            if (!this.PluginSettings.ShowDebugInfo)
                return;

            EventLog.WriteEntry(CROPPER_TWITPIC_EVENT_SOURCE_NAME,
                                msg,
                                EventLogEntryType.Information,
                                1);

        }

        private void LogDebugInfo(string msg, string details)
        {
            if (!this.PluginSettings.ShowDebugInfo)
                return;

            this.LogDebugInfo(msg, Encoding.GetEncoding("iso-8859-1").GetBytes(details));
        }

        private void LogDebugInfo(string msg, byte[] details)
        {
            if (!this.PluginSettings.ShowDebugInfo)
                return;

            EventLog.WriteEntry(CROPPER_TWITPIC_EVENT_SOURCE_NAME,
                                msg,
                                EventLogEntryType.Information,
                                1,
                                1,
                                details);
        }

        #endregion
    }
}
